﻿using System;
using System.IO;
using System.Windows.Media.Imaging;
using CBR.Core.Helpers;
using CBR.Core.Models;
using CBR.Core.Services;

namespace CBR.ViewModels
{
    public class CatalogViewModel : ViewModelBase
    {
        #region ----------------CONSTRUCTOR----------------

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="inf"></param>
        public CatalogViewModel(Catalog data)
		{
			Data = data;
            //Mediator.Instance.RegisterHandler<WorkspaceInfo>(ViewModelMessages.SettingsChanged, HandleSettingsChange);

		}

		/// <summary>
		/// Child classes can override this method to perform clean-up logic, such as removing event handlers.
		/// </summary>
		override protected void OnDispose()
		{
            base.OnDispose();
			//Mediator.Instance.UnregisterHandler<WorkspaceInfo>(ViewModelMessages.SettingsChanged, HandleSettingsChange);
		}

		#endregion

		#region ----------------PROPERTIES----------------

        private Uri _DefaultCoverUri = new Uri("pack://application:,,,/Resources/Images/book128.png");

		new public Catalog Data
		{
            get { return base.Data as Catalog; }
			set
			{
				base.Data = value;

				BookCount = value.BookInfoFilePath.Count;
				Title = value.Title;
				Description = value.Description;
				IsShared = value.IsShared;
				_CoverImage = null;
				RaisePropertyChanged("CoverImage");
			}
        }

		private int _BookCount;
		public int BookCount
		{
			get { return _BookCount; }
			set
			{
				if (_BookCount != value)
				{
					_BookCount = value;
					RaisePropertyChanged("BookCount");
				}
			}
        }

		private string _Title;
		public string Title
		{
            get
			{
				if (string.IsNullOrEmpty(_Title))
					_Title = Path.GetFileName(Data.CatalogFilePath);

				return _Title;
			}
            set
            {
                if (_Title != value)
                {
                    _Title = value;
                    RaisePropertyChanged("Title");
                }
            }
        }

		private string _Description;
        public string Description
        {
			get
			{
				if (string.IsNullOrEmpty(_Description))
					_Description = this.Title;

				return _Description;
			}
            set
            {
				if (_Description != value)
                {
					_Description = value;
                    RaisePropertyChanged("Description");
                }
            }
        }

		private bool _IsShared;
        public bool IsShared
        {
			get { return _IsShared; }
            set
            {
				if (_IsShared != value)
                {
					_IsShared = value;
                    RaisePropertyChanged("IsShared");
                }
            }
        }

		//private Uri _CoverUri;
		//public Uri CoverUri
		//{
		//	get
		//	{
		//		return _CoverUri;
		//	}
		//	set
		//	{
		//		if (_CoverUri != value)
		//		{
		//			_CoverUri = value;
		//			RaisePropertyChanged("CoverUri");
		//		}
		//	}
		//}

		private BitmapImage _CoverImage = null;
		/// <summary>
		/// the image 
		/// </summary>
		public BitmapImage CoverImage
		{
			get
			{
				if (_CoverImage == null)
				{
					_CoverImage = CatalogService.Instance.GenerateCatalogCover(Data);
					//_CoverUri = Data.CoverUri;
				}
				return _CoverImage;
			}
			set { _CoverImage = value; }
		}

        #endregion
    }
}
